-- DDR SDRAM Controller for test

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
library XilinxCoreLib;   
-- pragma translate_on

library work;
use work.ddr_controller_pack.all;

entity DDRtest is
	port(
		clk : in std_logic;
		reset : in std_logic;
		lcd_d : inout std_logic_vector(7 downto 4);
		lcd_rs : out std_logic;
		lcd_rw : out std_logic;
		lcd_e : out std_logic;
		sf_ce0 : out std_logic;
		rot_a : in std_logic;
		rot_b : in std_logic;
		rot_center : in std_logic;
		btn_east : in std_logic; -- f[^ݗp
		btn_west : in std_logic; -- f[^ǂݏop
		led : out std_logic_vector(7 downto 0);

		sd_a : out std_logic_vector(12 downto 0);
		sd_dq : inout std_logic_vector(15 downto 0);
		sd_ba : out std_logic_vector(1 downto 0);
		sd_ras : out std_logic;
		sd_cas : out std_logic;
		sd_we : out std_logic;
		sd_udm : out std_logic;
		sd_ldm : out std_logic;
		sd_udqs : inout std_logic;
		sd_ldqs : inout std_logic;
		sd_cs : out std_logic;
		sd_cke : out std_logic;
		sd_ck_n : out std_logic;
		sd_ck_p : out std_logic;
		sd_ck_fb : in std_logic
		);
end DDRtest;

architecture RTL of DDRtest is
component Test_LCD4HexDisp
	port(
		clk : in std_logic;
		reset : in std_logic;
		lcd_d : inout std_logic_vector(7 downto 4);
		lcd_rs : out std_logic;
		lcd_rw : out std_logic;
		lcd_e : out std_logic;
		sf_ce0 : out std_logic;
		rot_a : in std_logic;
		rot_b : in std_logic;
		rot_center : in std_logic;
		DDR_read_data : in std_logic_vector(15 downto 0);
		DDR_write_data : out std_logic_vector(15 downto 0)
	);
end component;
	COMPONENT dcm100
	PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		CLK2X_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;
component ddr_sdram_cont
	port(
		clk_in : in std_logic; -- NbN
		clk_out : out std_logic; -- NbNo
		clk_fb : in std_logic; -- NbNtB[hobN
		reset : in std_logic;
		input_data : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		input_mask : in std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
		read_write : in std_logic;
		output_data : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		input_address : in std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		addr_fifo_wren : in std_logic; -- AhXpFIFȌ݃Cl[u
		wrdata_fifo_wren : in std_logic; -- FIFȌ݃Cl[u
		addr_fifo_full : out std_logic;
		wrdata_fifo_full : out std_logic;
		rddata_valid : out std_logic; -- ǂݏof[^L
		initialize_end : out std_logic;
		
		ddr_clk : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0);
		ddr_clkb : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		ddr_cke : out std_logic;
		ddr_dqs : inout std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0) := (others => 'Z');
		ddr_dq : inout std_logic_vector(DDR_DATA_WIDTH-1 downto 0) := (others => 'Z');
		ddr_csb : out std_logic;
		ddr_rasb : out std_logic;
		ddr_casb : out std_logic;
		ddr_web : out std_logic;
		ddr_dm : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
		ddr_ba : out std_logic_vector(1 downto 0);
		ddr_address : out std_logic_vector(DDR_ADDRESS_WIDTH-1 downto 0)
	);
end component;
component SWDiv
	generic(
		clk_frequency : integer := 50000 -- KHz
	);
	port(
		clk, reset : in std_logic;
		sw_ena : out std_logic	
	);
end component;

signal clk50, clk100 : std_logic;
signal logic0, logic1 : std_logic;
signal dcm_locked : std_logic;
signal input_data : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal input_mask : std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
signal read_write : std_logic;
signal input_address : std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
signal addr_fifo_wren : std_logic; -- AhXpFIFȌ݃Cl[u
signal wrdata_fifo_wren : std_logic; -- FIFȌ݃Cl[u
signal addr_fifo_full : std_logic;
signal wrdata_fifo_full : std_logic;
signal rddata_valid : std_logic; -- ǂݏof[^L
signal initialize_end : std_logic;
signal ddr_clk : std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0);
signal ddr_clkb : std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
signal ddr_cke : std_logic;
signal ddr_dqs : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0) := (others => 'Z');
signal ddr_dq : std_logic_vector(DDR_DATA_WIDTH-1 downto 0) := (others => 'Z');
signal ddr_csb : std_logic;
signal ddr_rasb : std_logic;
signal ddr_casb : std_logic;
signal ddr_web : std_logic;
signal ddr_dm : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal ddr_ba : std_logic_vector(1 downto 0);
signal ddr_address : std_logic_vector(DDR_ADDRESS_WIDTH-1 downto 0);
signal DDR_write_data, DDR_read_data : std_logic_vector(15 downto 0);
signal sw_ena : std_logic;
signal reset_ddr_cont : std_logic;
signal output_data, loutdata : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal read_write_node, addr_fifo_wren_node, wrdata_fifo_wren_node : std_logic;
begin
	logic0 <= '0';
	logic1 <= '1';
	
	dcm100_inst : dcm100 port map(
		clkin_in => clk,
		rst_in => logic0,
		clk0_out => clk50,
		clk2X_out => clk100,
		locked_out => dcm_locked
	);
	
	reset_ddr_cont <= reset or (not dcm_locked);
	ddr_sdram_cont_inst : ddr_sdram_cont port map(
		clk_in => clk100,
		clk_out => open,
		clk_fb => sd_ck_fb,
		reset => reset_ddr_cont,
		input_data => input_data,
		input_mask => input_mask,
		read_write => read_write,
		output_data => output_data,
		input_address => input_address,
		addr_fifo_wren => addr_fifo_wren,
		wrdata_fifo_wren => wrdata_fifo_wren,
		addr_fifo_full => addr_fifo_full,
		wrdata_fifo_full => wrdata_fifo_full,
		rddata_valid => rddata_valid,
		initialize_end => initialize_end,
		
		ddr_clk => ddr_clk,
		ddr_clkb => ddr_clkb,
		ddr_cke => ddr_cke,
		ddr_dqs(1) => sd_udqs,
		ddr_dqs(0) => sd_ldqs,
		ddr_dq => sd_dq,
		ddr_csb => ddr_csb,
		ddr_rasb => ddr_rasb,
		ddr_casb => ddr_casb,
		ddr_web => ddr_web,
		ddr_dm => ddr_dm,
		ddr_ba => ddr_ba,
		ddr_address => ddr_address
	);
--	input_data(INTERFACE_DATA_WIDTH-1 downto 16) <= (others => '0');
	input_data(INTERFACE_DATA_WIDTH-1 downto 16) <= not DDR_write_data;
	input_data(15 downto 0) <= DDR_write_data;
	input_mask <= (others => '0');
	input_address <= (others => '0');
	process(clk100, reset) begin -- output_datab`
		if reset='1' then
			loutdata <= (others => '0');
		elsif clk100'event and clk100='1' then
			if rddata_valid='1' then
				loutdata <= output_data;
			end if;
		end if;
	end process;
	DDR_read_data <= loutdata(15 downto 0);
	
	sd_ck_n <= ddr_clkb(0);
	sd_ck_p <= ddr_clk(0);
	sd_cke <= ddr_cke;
	sd_cs <= ddr_csb;
	sd_a <= ddr_address;
	sd_ras <= ddr_rasb;
	sd_cas <= ddr_casb;
	sd_we <= ddr_web;
	sd_udm <= ddr_dm(1);
	sd_ldm <= ddr_dm(0);
	sd_ba <= ddr_ba;
	
	Test_LCD4HexDisp_inst : Test_LCD4HexDisp port map(
		clk => clk50,
		reset => reset,
		lcd_d => lcd_d,
		lcd_rs => lcd_rs,
		lcd_rw => lcd_rw,
		lcd_e => lcd_e,
		sf_ce0 => sf_ce0,
		rot_a => rot_a,
		rot_b => rot_b,
		rot_center => rot_center,
		DDR_write_data => DDR_write_data,
		DDR_read_data => DDR_read_data
	);
	
	swdiv_inst : swdiv
	-- pragma translate_off
	generic map(
		clk_frequency => 10
	)
	-- pragma translate_on
	port map(
		clk => clk100,
		reset => reset,
		sw_ena => sw_ena
	);
	
	process(reset, clk100) begin -- write pulse
		if reset='1' then
			read_write_node <= '0';
			addr_fifo_wren_node <= '0';
			wrdata_fifo_wren_node <= '0';
		elsif clk100'event and clk100='1' then
			if btn_east='1' and sw_ena='1' then -- ݃{^
				read_write_node <= '0';
				addr_fifo_wren_node <= '1';
				wrdata_fifo_wren_node <= '1';
			elsif btn_west='1' and sw_ena='1' then -- ǂݏo{^
				read_write_node <= '1';
				addr_fifo_wren_node <= '1';
			else
				addr_fifo_wren_node <= '0';
				wrdata_fifo_wren_node <= '0';
			end if;
		end if;
	end process;
	read_write <= read_write_node
	-- pragma translate_off
	after 1 ns
	-- pragma translate_on
	;
	addr_fifo_wren <= addr_fifo_wren_node
	-- pragma translate_off
	after 1 ns
	-- pragma translate_on
	;
	wrdata_fifo_wren <= wrdata_fifo_wren_node
	-- pragma translate_off
	after 1 ns
	-- pragma translate_on
	;
	led <= DDR_read_data(7 downto 0);
end RTL;

		