-- DCM module
-- DCMclkrxȊÕNbN𐶐

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
library work;
use work.ddr_controller_pack.all;

entity dcm_module is
	port(
		sysclk : in std_logic;
		sysclk_fb : in std_logic;
		reset : in std_logic;
		ddr_clk : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		ddr_clkb : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		clk : out std_logic;
		clk90 : out std_logic;
		clkx : out std_logic;
		clk270 : out std_logic;
		clk1_16 : out std_logic;
		dcm_lock : out std_logic
	);
end dcm_module;

architecture RTL of dcm_module is
component DCM
-- pragma translate_off
    generic ( 
             DFS_FREQUENCY_MODE : string := "LOW";
             CLKFX_DIVIDE : integer := 2;
	          CLKFX_MULTIPLY : integer := 3 ;
             CLK_FEEDBACK : string := "1X";
			 CLKDV_DIVIDE          : real := 16.0
--             STARTUP_WAIT : boolean := FALSE
            );
-- pragma translate_on

    port ( CLKIN     : in  std_logic;
           CLKFB     : in  std_logic;
           DSSEN     : in  std_logic;
           PSINCDEC  : in  std_logic;
           PSEN      : in  std_logic;
           PSCLK     : in  std_logic;
           RST       : in  std_logic;
           CLK0      : out std_logic;
           CLK90     : out std_logic;
           CLK180    : out std_logic;
           CLK270    : out std_logic;
           CLK2X     : out std_logic;
           CLK2X180  : out std_logic;
           CLKDV     : out std_logic;
           CLKFX     : out std_logic;
           CLKFX180  : out std_logic;
           LOCKED    : out std_logic;
           PSDONE    : out std_logic;
           STATUS    : out std_logic_vector(7 downto 0)
          );
end component;
component BUFG 
  port (
	I   : in std_logic;
	O    : out std_logic
	); 
end component;
component FDDRRSE
	port(
		q : out std_logic;
		d0 : in std_logic;
		d1 : in std_logic;
		c0 : in std_logic;
		c1 : in std_logic;
		ce : in std_logic;
		r : in std_logic;
		s : in std_logic
	);
end component;

attribute DLL_FREQUENCY_MODE : string; 
attribute DUTY_CYCLE_CORRECTION : string;
attribute CLKDV_DIVIDE          : string; 
attribute PHASE_SHIFT : integer;
attribute CLKOUT_PHASE_SHIFT : string;
attribute clkin_period : string;
attribute STARTUP_WAIT : string;
attribute FACTORY_JF : string;

attribute clkin_period of DCM_INST1 : label is "10";
attribute DLL_FREQUENCY_MODE of DCM_INST1    : label is "LOW";
attribute DUTY_CYCLE_CORRECTION of DCM_INST1 : label is "TRUE";
attribute CLKDV_DIVIDE of DCM_INST1          : label is "16.0";
attribute PHASE_SHIFT of DCM_INST1 : label is 0;
attribute CLKOUT_PHASE_SHIFT of DCM_INST1 : label is "NONE"; 
attribute STARTUP_WAIT of DCM_INST1 : label is "FALSE";
attribute FACTORY_JF of DCM_INST1 : label is "FFFF";

attribute clkin_period of DCM_INST_EXT : label is "10";
attribute DLL_FREQUENCY_MODE of DCM_INST_EXT    : label is "LOW";
attribute DUTY_CYCLE_CORRECTION of DCM_INST_EXT : label is "TRUE";
attribute CLKDV_DIVIDE of DCM_INST_EXT          : label is "16.0";
attribute PHASE_SHIFT of DCM_INST_EXT : label is 0;
attribute CLKOUT_PHASE_SHIFT of DCM_INST_EXT : label is "NONE"; 
attribute STARTUP_WAIT of DCM_INST_EXT : label is "FALSE";
attribute FACTORY_JF of DCM_INST_EXT : label is "FFFF";

signal vcc, gnd : std_logic;
signal clk_bufg, clk_node, clk90_node, clkx_node, clk270_node, clk1_16_node, dcm1_lock : std_logic;
signal clkx_bufg, clk90_bufg, clk270_bufg, clk1_16_bufg : std_logic;
signal clk_ext_node, clk_ext_bufg, clkx_ext_bufg : std_logic;
signal locked_ext : std_logic;
begin
	vcc <= '1'; gnd <= '0';
	
	DCM_INST_EXT : dcm port map(
		clkin => sysclk,
		clkfb => sysclk_fb,
		dssen => gnd,
		psincdec => gnd,
		psen => gnd,
		psclk => gnd,
		rst => gnd, -- ZbgƂDCM̃bNOȂ悤gndɂĂ
		clk0 => clk_ext_node,
		locked => locked_ext
	);
	
	ddr_clk_bufg : bufg port map(
		i => clk_ext_node,
		o => clk_ext_bufg
	);
	
	DCM_INST1 : dcm port map(
		clkin => sysclk,
		clkfb => clk_bufg,
		dssen => gnd,
		psincdec => gnd,
		psen => gnd,
		psclk => gnd,
		rst => gnd, -- ZbgƂDCM̃bNOȂ悤gndɂĂ
		clk0 => clk_node,
		clk90 => clk90_node,
		clk180 => clkx_node,
		clk270 => clk270_node,
		clk2x => open,
		clk2x180 => open,
		clkdv => clk1_16_node,
		clkfx => open,
		clkfx180 => open,
		locked => dcm1_lock,
		psdone => open,
		status => open
	);
	
	clk_bufg_inst : bufg port map(
		i => clk_node,
		o => clk_bufg
	);
	clk <= clk_bufg;
	
	clk90_bufg_inst : bufg port map(
		i => clk90_node,
		o => clk90_bufg
	);
	clk90 <= clk90_bufg;
	
--	clkx_bufg_inst : bufg port map(
--		i => clkx_node,
--		o => clkx_bufg
--	);
--	clkx <= clkx_bufg;
	clkx <= not clk_bufg;
	
--	clk270_bufg_inst : bufg port map(
--		i => clk270_node,
--		o => clk270_bufg
--	);
--	clk270 <= clk270_bufg;
	clk270 <= not clk90_bufg;
	
--	clk1_16_bufg_inst : bufg port map(
--		i => clk1_16_node,
--		o => clk1_16_bufg
--	);
--	clk1_16 <= clk1_16_bufg;
	clk1_16 <= clk1_16_node;
	
	clkx_ext_bufg <= not clk_ext_bufg;
    ddrclk_gen : for i in 0 to QUANTITY_OF_CLK_OUTPUT-1 generate
		ddrclk_inst : FDDRRSE port map(
			q => ddr_clk(i),
			c0 => clk_ext_bufg,
			c1 => clkx_ext_bufg,
			ce => vcc,
			d0 => vcc,
			d1 => gnd,
			r => gnd,
			s => gnd
		);
		
		ddrclkb_inst : FDDRRSE port map(
			q => ddr_clkb(i),
			c0 => clk_ext_bufg,
			c1 => clkx_ext_bufg,
			ce => vcc,
			d0 => gnd,
			d1 => vcc,
			r => gnd,
			s => gnd
		);
	end generate ddrclk_gen;
	
	dcm_lock <= dcm1_lock and locked_ext;
end RTL;
