-- DDR SDRAM Controller
-- XAPP253ƔrăAhXFIFO{āA[hƃCg̃AhXFIFO1ɂB邱ƂɂāA[hƃCgEANZX̗D揇ʂ̖B
-- 
-- 2005.04.11

library ieee;
use ieee.std_logic_1164.all;

-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
library XilinxCoreLib;   
-- pragma translate_on

library work;
use work.ddr_controller_pack.all;

entity ddr_sdram_cont is
	port(
		clk_in : in std_logic; -- NbN
		clk_out : out std_logic; -- NbNo
		clk_fb : in std_logic; -- NbNtB[hobN
		reset : in std_logic;
		input_data : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		input_mask : in std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
		read_write : in std_logic;
		output_data : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		input_address : in std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		addr_fifo_wren : in std_logic; -- AhXpFIFȌ݃Cl[u
		wrdata_fifo_wren : in std_logic; -- FIFȌ݃Cl[u
		addr_fifo_full : out std_logic;
		wrdata_fifo_full : out std_logic;
		rddata_valid : out std_logic; -- ǂݏof[^L
		initialize_end : out std_logic;
		
		ddr_clk : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0);
		ddr_clkb : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		ddr_cke : out std_logic;
		ddr_dqs : inout std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0) := (others => 'Z');
		ddr_dq : inout std_logic_vector(DDR_DATA_WIDTH-1 downto 0) := (others => 'Z');
		ddr_csb : out std_logic;
		ddr_rasb : out std_logic;
		ddr_casb : out std_logic;
		ddr_web : out std_logic;
		ddr_dm : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
		ddr_ba : out std_logic_vector(1 downto 0);
		ddr_address : out std_logic_vector(DDR_ADDRESS_WIDTH-1 downto 0)
	);
end ddr_sdram_cont;

architecture RTL of ddr_sdram_cont is
component dcm_module
	port(
		sysclk : in std_logic;
		sysclk_fb : in std_logic;
		reset : in std_logic;
		ddr_clk : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		ddr_clkb : out std_logic_vector(QUANTITY_OF_CLK_OUTPUT-1 downto 0) ;
		clk : out std_logic;
		clk90 : out std_logic;
		clkx : out std_logic;
		clk270 : out std_logic;
		clk1_16 : out std_logic;
		dcm_lock : out std_logic
	);
end component;
component controller
	port(
		clk : in std_logic;
		clkx : in std_logic;
		clk1_16 : in std_logic; -- 16NbN
		reset : in std_logic;
		dcm_lock : in std_logic;
		address : in std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		read_writex : in std_logic; -- 1 - read, 0 - write
		next_address : in std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		next_read_writex : in std_logic; -- 1 - read, 0 - write
		addr_fifo_empty : in std_logic;
		addr_fifo_almost_empty : in std_logic;
		addr_fifo_rden : out std_logic;
		wrdata_fifo_empty : in std_logic;
		wrdata_fifo_almost_empty : in std_logic;
		wrdata_fifo_rden : out std_logic;
		ddr_rasb : out std_logic;
		ddr_casb : out std_logic;
		ddr_web : out std_logic;
		ddr_ba : out std_logic_vector(1 downto 0);
		ddr_address : out std_logic_vector(DDR_ADDRESS_WIDTH-1 downto 0);
		ddr_cke : out std_logic;
		ddr_csb : out std_logic;
		dqs_enable : out std_logic;
		dqs_reset : out std_logic;
		write_timing : out std_logic;
		read_timing : out std_logic;
		burst_read : out std_logic;
		initialize_end : out std_logic
	);
end component;
component read_data_module
	port(
		reset : in std_logic;
		clk : in std_logic;
		clkx : in std_logic;
		read_timing : in std_logic;
		burst_read : in std_logic;
		rddata_fifo_data : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		rddata_fifo_wren : out std_logic;
		rddata_fifo_full : in std_logic;
		rddata_fifo_almost_full : in std_logic;
		
		ddr_dq : in std_logic_vector(DDR_DATA_WIDTH-1 downto 0);
		ddr_dqs : in std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0)
	);
end component;
component write_data_module
	port(
		reset : in std_logic;
		clk : in std_logic;
		clk90 : in std_logic;
		clkx : in std_logic;
		clk270 : in std_logic;
		wrdata_fifo_rden : in std_logic;
		wrdata_fifo_data : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		wrdata_fifo_mask : in std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
		dqs_enable : in std_logic;
		dqs_reset : in std_logic;
		
		ddr_dqs : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
		ddr_dq : out std_logic_vector(DDR_DATA_WIDTH-1 downto 0);
		ddr_dm : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0)
	);
end component;
component addr_fifo
	port(
		clk : in std_logic;
		reset : in std_logic;
		din : in std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		read_write : in std_logic;
		wr_en : in std_logic;
		rd_en : in std_logic;
		dout : out std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		rw_out : out std_logic; -- read/write output
		full : out std_logic;
		empty : out std_logic;
		next_dout : out std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
		next_rw_out : out std_logic; -- next read/write output
		almost_empty : out std_logic;
		almost_full : out std_logic
	);
end component;
component wrdata_fifo
	port(
		clk : in std_logic;
		reset : in std_logic;
		din : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		maskin : in std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
		rd_en : in std_logic;
		wr_en : in std_logic;
		full : out std_logic;
		almost_full : out std_logic;
		empty : out std_logic;
		almost_empty : out std_logic;
		dout : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		maskout : out std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0)
	);
end component;
component rddata_fifo
	port(
		clk : in std_logic;
		reset : in std_logic;
		din : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		rd_en : in std_logic;
		wr_en : in std_logic;
		full : out std_logic;
		almost_full : out std_logic;
		empty : out std_logic;
		almost_empty : out std_logic;
		dout : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0)
	);
end component;

signal clk, clk90, clkx, clk270, clk1_16, dcm_lock : std_logic;
signal address, next_address : std_logic_vector(USER_INPUT_ADDRESS_WIDTH-1 downto 0);
signal read_writex, next_read_writex : std_logic;
signal addr_fifo_empty, addr_fifo_almost_empty, addr_fifo_rden : std_logic;
signal wrdata_fifo_empty, wrdata_fifo_almost_empty, wrdata_fifo_rden : std_logic;
signal dqs_enable, dqs_reset, write_timing, read_timing, burst_read : std_logic;
signal wrdata_fifo_data : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal wrdata_fifo_mask : std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
signal wrdata_fifo_almost_full : std_logic;
signal rddata_fifo_rden : std_logic;
signal rddata_fifo_empty ,rddata_fifo_almost_empty : std_logic;
signal rddata_fifo_data : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal rddata_fifo_wren, rddata_fifo_full, rddata_fifo_almost_full : std_logic;
signal ie, iel : std_logic;
begin
	dcm_module_inst : dcm_module port map(
		sysclk => clk_in,
		sysclk_fb => clk_fb,
		reset => reset,
		ddr_clk => ddr_clk,
		ddr_clkb => ddr_clkb,
		clk => clk,
		clk90 => clk90,
		clkx => clkx,
		clk270 => clk270, 
		clk1_16 => clk1_16,
		dcm_lock => dcm_lock
	);
		
	
	controller_inst : controller port map(
		clk => clk,
		clkx => clkx,
		clk1_16 => clk1_16,
		reset => reset,
		dcm_lock => dcm_lock,
		address => address,
		read_writex => read_writex,
		next_address => next_address,
		next_read_writex => next_read_writex,
		addr_fifo_empty => addr_fifo_empty,
		addr_fifo_almost_empty => addr_fifo_almost_empty,
		addr_fifo_rden => addr_fifo_rden,
		wrdata_fifo_empty => wrdata_fifo_empty,
		wrdata_fifo_almost_empty => wrdata_fifo_almost_empty,
		wrdata_fifo_rden => wrdata_fifo_rden,
		ddr_rasb => ddr_rasb,
		ddr_casb => ddr_casb,
		ddr_web => ddr_web,
		ddr_ba => ddr_ba,
		ddr_address => ddr_address,
		ddr_cke => ddr_cke,
		ddr_csb => ddr_csb,
		dqs_enable => dqs_enable,
		dqs_reset => dqs_reset,
		write_timing => write_timing,
		read_timing => read_timing,
		burst_read => burst_read,
		initialize_end => ie
	);
	process(reset, clk) begin
		if reset='1' then
			iel <= '0';
		elsif clk'event and clk='1' then
			iel <= ie;
		end if;
	end process;
	initialize_end <= iel;
	
	read_data_module_inst : read_data_module port map(
		reset => reset,
		clk => clk,
		clkx => clkx,
		read_timing => read_timing,
		burst_read => burst_read,
		rddata_fifo_data => rddata_fifo_data,
		rddata_fifo_wren => rddata_fifo_wren,
		rddata_fifo_full => rddata_fifo_full,
		rddata_fifo_almost_full => rddata_fifo_almost_full,
		ddr_dq => ddr_dq,
		ddr_dqs => ddr_dqs
	);
	
	write_data_module_inst : write_data_module port map(
		reset => reset,
		clk => clk,
		clk90 => clk90,
		clkx => clkx,
		clk270 =>clk270,
		wrdata_fifo_rden => wrdata_fifo_rden,
		wrdata_fifo_data => wrdata_fifo_data,
		wrdata_fifo_mask => wrdata_fifo_mask,
		dqs_enable => dqs_enable,
		dqs_reset => dqs_reset,
		ddr_dqs => ddr_dqs,
		ddr_dq => ddr_dq,
		ddr_dm => ddr_dm
	);
	
	addr_fifo_inst : addr_fifo port map(
		clk => clk,
		reset => reset,
		din => input_address,
		read_write => read_write,
		wr_en => addr_fifo_wren,
		rd_en => addr_fifo_rden,
		dout => address,
		rw_out => read_writex,
		full => addr_fifo_full,
		empty => addr_fifo_empty,
		next_dout => next_address,
		next_rw_out => next_read_writex,
		almost_empty => addr_fifo_almost_empty,
		almost_full => open
	);
	
	wrdata_fifo_inst : wrdata_fifo port map(
		clk => clk,
		reset => reset,
		din => input_data,
		maskin => input_mask,
		rd_en => wrdata_fifo_rden,
		wr_en => wrdata_fifo_wren,
		full => wrdata_fifo_full,
		almost_full => wrdata_fifo_almost_full,
		empty => wrdata_fifo_empty,
		almost_empty => wrdata_fifo_almost_empty,
		dout => wrdata_fifo_data,
		maskout => wrdata_fifo_mask
	);
		
	rddata_fifo_inst : rddata_fifo port map(
		clk => clk,
		reset => reset,
		din => rddata_fifo_data,
		rd_en => rddata_fifo_rden,
		wr_en => rddata_fifo_wren,
		full => rddata_fifo_full,
		almost_full => rddata_fifo_almost_full,
		empty => rddata_fifo_empty,
		almost_empty => rddata_fifo_almost_empty,
		dout => output_data
	);
    rddata_fifo_rden <= '1' when rddata_fifo_empty='0' else '0'; -- emptyłȂ́A[hf[^ǂݏo
	rddata_valid <= rddata_fifo_rden;
	clk_out <= clk;
end RTL;
