-- rddata_fifo
-- ǂݏof[^Ă

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
library work;
use work.ddr_controller_pack.all;

entity rddata_fifo is
	port(
		clk : in std_logic;
		reset : in std_logic;
		din : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		rd_en : in std_logic;
		wr_en : in std_logic;
		full : out std_logic;
		almost_full : out std_logic;
		empty : out std_logic;
		almost_empty : out std_logic;
		dout : out std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0)
	);
end rddata_fifo;

architecture RTL of rddata_fifo is
component RAM16X1D
  generic (
    INIT : bit_vector := X"0000"
    );

  port (
    DPO : out std_ulogic;
    SPO : out std_ulogic;

    A0    : in std_ulogic;
    A1    : in std_ulogic;
    A2    : in std_ulogic;
    A3    : in std_ulogic;
    D     : in std_ulogic;
    DPRA0 : in std_ulogic;
    DPRA1 : in std_ulogic;
    DPRA2 : in std_ulogic;
    DPRA3 : in std_ulogic;
    WCLK  : in std_ulogic;
    WE    : in std_ulogic
    );
end component;

signal wp, rp : std_logic_vector(3 downto 0);
signal data_node, out_data : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal outff_flag : std_logic;
signal fifo_full, fifo_empty : std_logic;
signal inner_rd_en : std_logic;
signal we :std_logic;
begin
	FIFO_DATA_GEN : for i in INTERFACE_DATA_WIDTH-1 downto 0 generate
		RAM16_INST : RAM16X1D port map(
			DPO => data_node(i),
			SPO => open,
			A0 => wp(0),
			A1 => wp(1),
			A2 => wp(2),
			A3 => wp(3),
			D => din(i),
			DPRA0 => rp(0),
			DPRA1 => rp(1),
			DPRA2 => rp(2),
			DPRA3 => rp(3),
			WCLK => clk,
			WE => we
		);
	end generate FIFO_DATA_GEN;
	we <= wr_en and not fifo_full;

	process(reset, clk) begin -- wp, rp
		if reset='1' then
			wp <= (others => '0');
			rp <= (others => '0');
		elsif clk'event and clk='1' then
			if wr_en='1' and fifo_full='0' then -- wr_en1ŁAFIFOtłȂƂɏ
				wp <= wp + 1;
			end if;
			if inner_rd_en='1' and fifo_empty='0' then -- rd_en1ŁAFIFOemptyoȂɓǂݏo
				rp <= rp + 1;
			end if;
		end if;
	end process;
	fifo_full <= '1' when rp-1 = wp else '0'; -- rp1wptB1GggȂ傤Ȃ
	full <= fifo_full;
	almost_full <= '1' when rp-2 = wp else '0';
	fifo_empty <= '1' when rp = wp else '0'; -- rp = wp̎AFIFO͋
	
	process(reset, clk) begin -- out_data
		if reset='1' then
			out_data <= (others => '0');
		elsif clk'event and clk='1' then
			if fifo_empty='0' and (not(rd_en='0' and outff_flag='1')) then
			-- FIFOłȂAoutff1œǂݏoȂȊOoutffFIFO̓eǂݏoB
				out_data <= data_node;
			end if;
		end if;
	end process;
	inner_rd_en <= '1' when fifo_empty='0' and (not(rd_en='0' and outff_flag='1')) else '0';
	
	process(reset, clk) begin -- flag
		if reset='1' then
			outff_flag <= '0';
		elsif clk'event and clk='1' then
			if inner_rd_en='1' then
				outff_flag <= '1';
			elsif rd_en='1' then
				outff_flag <= '0';
			end if;
		end if;
	end process;
	dout <= out_data;
	empty <= not outff_flag;
	almost_empty <= '1' when outff_flag='0' or (outff_flag='1' and fifo_empty='1') else '0';
end RTL;
