-- Write Data Module
-- DQSDQADMo͂
-- DQSclk90ɘAĕωB
-- DQWRITsꂽclk(0xʑj̔NbNォCl[uɂB

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
library work;
use work.ddr_controller_pack.all;

entity write_data_module is
	port(
		reset : in std_logic;
		clk : in std_logic;
		clk90 : in std_logic;
		clkx : in std_logic;
		clk270 : in std_logic;
		wrdata_fifo_rden : in std_logic;
		wrdata_fifo_data : in std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
		wrdata_fifo_mask : in std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
		dqs_enable : in std_logic;
		dqs_reset : in std_logic;
		
		ddr_dqs : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
		ddr_dq : out std_logic_vector(DDR_DATA_WIDTH-1 downto 0);
		ddr_dm : out std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0)
	);
end write_data_module;

architecture RTL of write_data_module is
component FDDRRSE
	port(
		q : out std_logic;
		d0 : in std_logic;
		d1 : in std_logic;
		c0 : in std_logic;
		c1 : in std_logic;
		ce : in std_logic;
		r : in std_logic;
		s : in std_logic
	);
end component;
component FDDRCPE
	port(
		q : out std_logic;
		d0 : in std_logic;
		d1 : in std_logic;
		c0 : in std_logic;
		c1 : in std_logic;
		ce : in std_logic;
		clr : in std_logic;
		pre : in std_logic
	);
end component;

signal wrdata_1d, wrdata_2d : std_logic_vector(INTERFACE_DATA_WIDTH-1 downto 0);
signal wrdata_3d_half : std_logic_vector(INTERFACE_DATA_WIDTH/2-1 downto 0);
signal wrmask_1d, wrmask_2d : std_logic_vector(INTERFACE_MASK_WIDTH-1 downto 0);
signal wrmask_3d_half : std_logic_vector(INTERFACE_MASK_WIDTH/2-1 downto 0);
signal ddr_out, out_tri : std_logic_vector(DDR_DATA_WIDTH-1 downto 0);
signal dqs_out : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal gnd, vcc : std_logic;
signal dqs_enable_1d : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal dqs_enable_2d : std_logic_vector(DDR_DQS_DM_WIDTH*8-1 downto 0);
signal dqs_reset_1d : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal dqs_reset_2d_dqtri : std_logic_vector(DDR_DQS_DM_WIDTH*8-1 downto 0);
signal dqs_reset_2d : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal dqs_tri_enable_2dx : std_logic_vector(DDR_DQS_DM_WIDTH-1 downto 0);
signal not_dqs_enable_2d : std_logic_vector(DDR_DQS_DM_WIDTH*8-1 downto 0);
signal dq_tri_d0, dq_tri_d1 : std_logic_vector(DDR_DQS_DM_WIDTH*8-1 downto 0);
attribute syn_maxfan : integer;
attribute syn_maxfan of dqs_reset_2d :signal is 1;
attribute syn_maxfan of dqs_reset_2d_dqtri :signal is 1;
attribute syn_maxfan of dqs_enable_2d :signal is 1;
attribute syn_maxfan of dqs_reset_1d :signal is 1;
attribute syn_maxfan of dqs_enable_1d :signal is 1;
attribute syn_maxfan of dqs_tri_enable_2dx :signal is 1;
begin
	gnd <= '0';
	vcc <= '1';
	
	process(reset, clk) begin
		if reset='1' then
			wrdata_1d <= (others => '0');
			wrmask_1d <= (others => '0');
			wrdata_2d <= (others => '0');
			wrmask_2d <= (others => '0');
			wrdata_3d_half <= (others => '0');
			wrmask_3d_half <= (others => '0');
		elsif clk'event and clk='1' then
--			if wrdata_fifo_rden='1' then
				wrdata_1d <= wrdata_fifo_data;
				wrmask_1d <= wrdata_fifo_mask;
				wrdata_2d <= wrdata_1d;
				wrmask_2d <= wrmask_1d;
				wrdata_3d_half <= wrdata_2d(INTERFACE_DATA_WIDTH-1 downto INTERFACE_DATA_WIDTH/2);
				wrmask_3d_half <= wrmask_2d(INTERFACE_MASK_WIDTH-1 downto INTERFACE_MASK_WIDTH/2);
--			end if;
		end if;
	end process;
	process(reset, clkx) begin
		if reset='1' then
			dqs_enable_1d <= (others => '0');
			dqs_enable_2d <= (others => '0');
			dqs_reset_1d <= (others => '0');
			dqs_reset_2d_dqtri <= (others => '0');
		elsif clkx'event and clkx='1' then
			
			for i in 0 to DDR_DQS_DM_WIDTH-1 loop
            	dqs_reset_1d(i) <= dqs_reset;
				dqs_enable_1d(i) <= dqs_enable;
			end loop;
			for i in 0 to DDR_DATA_WIDTH-1 loop
				dqs_enable_2d(i) <= dqs_enable_1d(i/8);
				dqs_reset_2d_dqtri(i) <= dqs_reset_1d(i/8);
			end loop;
		end if;
	end process;
	process(reset, clk90) begin
		if reset='1' then
			dqs_reset_2d <= (others => '0');
		elsif clk90'event and clk90='1' then
			dqs_reset_2d <= dqs_reset_1d;
		end if;
	end process;
	
	-- Instantiate Output DDR registers
	WRDATA_DDR_OUT_INST : for i in DDR_DATA_WIDTH-1 downto 0 generate
		WRDATA_DDR_OUT : FDDRCPE port map(
			q => ddr_out(i),
			d0 => wrdata_2d(i),
			d1 => wrdata_3d_half(i),
			c0 => clk,
			c1 => clkx,
			ce => dqs_enable_2d(i/8),
			clr => reset,
			pre => gnd
		);
	end generate WRDATA_DDR_OUT_INST;
	
	not_dqs_enable_2d <= not dqs_enable_2d;
	
	-- Insantiate three-state DDR registers
	dq_tri_d0 <= dqs_reset_2d_dqtri or not_dqs_enable_2d;
	dq_tri_d1 <= not_dqs_enable_2d;
	
	WRDATA_DDR_TRI_INST : for i in DDR_DATA_WIDTH-1 downto 0 generate
		WRDATA_DDR_TRI : FDDRRSE port map(
			q => out_tri(i),
			d0 => dq_tri_d0(i/8), -- DQS0ɂȂŏ̃NbN1Ȃ̂ŁAԍŏ̔NbNfBXG[u
			d1 => dq_tri_d1(i/8),
			c0 => clk,
			c1 => clkx,
			ce => vcc,
			r => reset,
--			s => not_dqs_enable_2d(i/8)
			s => gnd
		);
	end generate WRDATA_DDR_TRI_INST;
	
	-- inferr the 3-State buffer
	process(out_tri, ddr_out) begin
		for i in DDR_DATA_WIDTH-1 downto 0 loop
			if out_tri(i) = '1' then
				ddr_dq(i) <= 'Z';
--			elsif out_tri(i) = '0' then
			else
				ddr_dq(i) <= ddr_out(i);
			end if;
		end loop;
	end process;
	
	-- Instantiate Mask DDR registers
	WRDATA_DDR_MASK_INST : for i in DDR_DQS_DM_WIDTH-1 downto 0 generate
		WRDATA_DDR_MASK : FDDRCPE port map(
			q => ddr_dm(i),
			d0 => wrmask_2d(i),
			d1 => wrmask_3d_half(i),
			c0 => clk,
			c1 => clkx,
			ce => vcc,
			clr => reset,
			pre => gnd
		);
	end generate WRDATA_DDR_MASK_INST;
	
	-- Instantiate DQS DDR registers
	WRDATA_DDR_DQS_INST : for i in DDR_DQS_DM_WIDTH-1 downto 0 generate
		WRDATA_DDR_DQS : FDDRRSE port map(
			q => dqs_out(i),
			d0 => vcc,
			d1 => gnd,
			c0 => clk90,
			c1 => clk270,
			ce => vcc,
			r => dqs_reset_2d(i),
			s => gnd
		);
	end generate WRDATA_DDR_DQS_INST;

	process(reset, clk) begin -- dqs_enable_2dclkŃTvdqs̏o̓Cl[uƂ
		if reset='1' then
			dqs_tri_enable_2dx <= (others => '1');
		elsif clk'event and clk='1' then
			dqs_tri_enable_2dx <= not dqs_enable_2d(DDR_DQS_DM_WIDTH-1 downto 0);
		end if;
	end process;
	
	-- inferr the 3-State buffer
	process(dqs_tri_enable_2dx, dqs_out) begin
		for i in DDR_DQS_DM_WIDTH-1 downto 0 loop
			if dqs_tri_enable_2dx(i)='1' then
				ddr_dqs(i) <= 'Z';
--			elsif dqs_tri_enable_2dx(i)='0' then
			else
				ddr_dqs(i) <= dqs_out(i);
			end if;
		end loop;
	end process;
end RTL;
	