-- DDRtest_tb.vhd

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity testbench is
end testbench;

architecture testbench_arch of testbench is
component DDRtest
	port(
		clk : in std_logic;
		reset : in std_logic;
		lcd_d : inout std_logic_vector(7 downto 4);
		lcd_rs : out std_logic;
		lcd_rw : out std_logic;
		lcd_e : out std_logic;
		sf_ce0 : out std_logic;
		rot_a : in std_logic;
		rot_b : in std_logic;
		rot_center : in std_logic;
		btn_east : in std_logic; -- f[^ݗp
		btn_west : in std_logic; -- f[^ǂݏop

		sd_a : out std_logic_vector(12 downto 0);
		sd_dq : inout std_logic_vector(15 downto 0);
		sd_ba : out std_logic_vector(1 downto 0);
		sd_ras : out std_logic;
		sd_cas : out std_logic;
		sd_we : out std_logic;
		sd_udm : out std_logic;
		sd_ldm : out std_logic;
		sd_udqs : inout std_logic;
		sd_ldqs : inout std_logic;
		sd_cs : out std_logic;
		sd_cke : out std_logic;
		sd_ck_n : out std_logic;
		sd_ck_p : out std_logic;
		sd_ck_fb : in std_logic
		);
end component;
component MT46V16M16
    GENERIC (                                   -- Timing for -75Z CL2
        tCK       : TIME    :=  7.500 ns;
        tCH       : TIME    :=  3.375 ns;       -- 0.45*tCK
        tCL       : TIME    :=  3.375 ns;       -- 0.45*tCK
        tDH       : TIME    :=  0.500 ns;
        tDS       : TIME    :=  0.500 ns;
        tIH       : TIME    :=  0.900 ns;
        tIS       : TIME    :=  0.900 ns;
        tMRD      : TIME    := 15.000 ns;
        tRAS      : TIME    := 40.000 ns;
        tRAP      : TIME    := 20.000 ns;
        tRC       : TIME    := 65.000 ns;
        tRFC      : TIME    := 75.000 ns;
        tRCD      : TIME    := 20.000 ns;
        tRP       : TIME    := 20.000 ns;
        tRRD      : TIME    := 15.000 ns;
        tWR       : TIME    := 15.000 ns;
        addr_bits : INTEGER := 13;
        data_bits : INTEGER := 16;
        cols_bits : INTEGER :=  9
    );
    PORT (
        Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
        Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
        Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
        Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
        Clk   : IN    STD_LOGIC;
        Clk_n : IN    STD_LOGIC;
        Cke   : IN    STD_LOGIC;
        Cs_n  : IN    STD_LOGIC;
        Ras_n : IN    STD_LOGIC;
        Cas_n : IN    STD_LOGIC;
        We_n  : IN    STD_LOGIC;
        Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0)
    );
END component;
signal clk : std_logic := '1';
signal reset : std_logic;
signal lcd_d : std_logic_vector(7 downto 4);
signal lcd_rs : std_logic;
signal lcd_rw : std_logic;
signal lcd_e : std_logic;
signal sf_ce0 : std_logic;
signal rot_a : std_logic;
signal rot_b : std_logic;
signal rot_center : std_logic;
signal btn_east : std_logic; -- f[^ݗp
signal btn_west : std_logic; -- f[^ǂݏop

constant delay_time : time := 1.5 ns;
signal ddr_clk           : std_logic;
signal ddr_clkb          : std_logic;
signal ddr_dqs_fpga, ddr_dqs_sdram     : std_logic_vector(1  downto 0);
signal ddr_dq_sdram, ddr_dq_fpga : std_logic_vector(15 downto 0);
signal ddr_cke           : std_logic;
signal ddr_csb           : std_logic;
signal ddr_rasb          : std_logic;
signal ddr_casb          : std_logic;
signal ddr_web           : std_logic;
signal ddr_dm            : std_logic_vector(1  downto 0);
signal ddr_BA            : std_logic_vector(1  downto 0);
signal ddr_address       : std_logic_vector(12 downto 0);
signal reset_b, enable_o : std_logic;
signal cmd : std_logic_vector(2 downto 0);
signal sdram_clk : std_logic;
signal sdram_clkb : std_logic;
signal sdram_address : std_logic_vector(12 downto 0);
signal sdram_ba : std_logic_vector(1 downto 0);
signal sdram_cke : std_logic;
signal sdram_csb, sdram_rasb, sdram_casb, sdram_web : std_logic;
signal sdram_dm : std_logic_vector(1 downto 0);
signal ddr_clk_fb : std_logic;
begin
	CMD <= ( ddr_rasb & ddr_casb & ddr_web );
	
	process(ddr_clk, reset) begin
		if reset='1' then
			enable_o <= '0';
  		elsif ddr_clk'event and ddr_clk='1' then
			if CMD = "100" then -- Write
				enable_o <= '0';
			elsif CMD = "101" then -- Read
				enable_o <= '1';
			else
				enable_o <= enable_o;
			end if;
		end if;
	end process;
	-- During a READ
	ddr_dqs_fpga <= transport ddr_dqs_sdram after delay_time when enable_o = '1' else
	--ddr_dqs_fpga <= ddr_dqs_sdram when enable_o = '1' else
	                "ZZ";
--	ddr_dq_fpga <= transport ddr_dq_sdram after delay_time+4.5 ns when enable_o='1' else (others => 'Z');
	ddr_dq_fpga <= transport ddr_dq_sdram after delay_time when enable_o='1' else (others => 'Z');

	-- During a WRITE
	ddr_dqs_sdram <= transport ddr_dqs_fpga after delay_time when enable_o = '0' else
	--ddr_dqs_sdram <= ddr_dqs_fpga when enable_o = '0' else
	                 "ZZ";
	ddr_dq_sdram <= transport ddr_dq_fpga after delay_time when enable_o='0' else (others => 'Z');
	sdram_clk <= transport ddr_clk after delay_time;
	sdram_clkb <= transport ddr_clkb after delay_time;
	sdram_address <= transport ddr_address after delay_time;
	sdram_ba <= transport ddr_ba after delay_time;
	sdram_cke <= transport ddr_cke after delay_time;
	sdram_csb <= transport ddr_csb after delay_time;
	sdram_rasb <= transport ddr_rasb after delay_time;
	sdram_casb <= transport ddr_casb after delay_time;
	sdram_web <= transport ddr_web after delay_time;
	sdram_dm <= transport ddr_dm after delay_time;

	DDRtest_inst : DDRtest port map(
		clk => clk,
		reset => reset,
		lcd_d => lcd_d,
		lcd_rs => lcd_rs,
		lcd_rw => lcd_rw,
		lcd_e => lcd_e,
		sf_ce0 => sf_ce0,
		rot_a => rot_a,
		rot_b => rot_b,
		rot_center => rot_center,
		btn_east => btn_east,
		btn_west => btn_west,
		
		sd_a => ddr_address,
		sd_dq => ddr_dq_fpga,
		sd_ba => ddr_BA,
		sd_ras => ddr_rasb,
		sd_cas => ddr_casb,
		sd_we => ddr_web,
		sd_udm => ddr_dm(1),
		sd_ldm => ddr_dm(0),
		sd_udqs => ddr_dqs_fpga(1),
		sd_ldqs => ddr_dqs_fpga(0),
		sd_cs => ddr_csb,
		sd_cke => ddr_cke,
		sd_ck_n => ddr_clkb,
		sd_ck_p => ddr_clk,
		sd_ck_fb => ddr_clk_fb
	);
	ddr_clk_fb <= ddr_clk;
	
	MT46V16M16_inst : MT46V16M16 port map(
		Dq =>ddr_dq_sdram,
		Dqs => ddr_dqs_sdram,
		Addr => sdram_address,
		Ba => sdram_ba,
		Clk => sdram_clk,
		Clk_n => sdram_clkb,
		Cke => sdram_cke,
		Cs_n => sdram_csb,
		Ras_n => sdram_rasb,
		Cas_n => sdram_casb,
		We_n => sdram_web,
		Dm => sdram_dm
	);
	
	reset <= '1',
			'0' after 20 ns;
			
	clk <= not clk after 10 ns;
	
	process begin
	             rot_a <= '0';
	             rot_b <= '0';
	             rot_center <= '0';
	             btn_east <= '0';
	             btn_west <= '0';
	                -- -------------  Current Time:  101ns
                WAIT FOR 5000 ns;
                rot_a <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  401ns
                WAIT FOR 300 ns;
                rot_b <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  881ns
                WAIT FOR 480 ns;
                rot_a <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  1261ns
                WAIT FOR 380 ns;
                rot_b <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  1621ns
                WAIT FOR 360 ns;
                rot_center <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  2361ns
                WAIT FOR 740 ns;
                rot_center <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  2481ns
                WAIT FOR 120 ns;
                rot_b <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  2781ns
                WAIT FOR 300 ns;
                rot_center <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  3241ns
                WAIT FOR 460 ns;
                rot_b <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  3861ns
                WAIT FOR 620 ns;
                rot_center <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  4161ns
                WAIT FOR 300 ns;
                btn_east <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  5241ns
                WAIT FOR 1080 ns;
                btn_east <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  5821ns
                WAIT FOR 580 ns;
                btn_west <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  7121ns
                WAIT FOR 1300 ns;
                btn_west <= '0';
                -- -------------------------------------
				wait for 1000 us;
	end process;
end testbench_arch;
configuration DDRtest_sim_top_cfg of testbench is
	for testbench_arch
	end for;
end DDRtest_sim_top_cfg;
